/*
 * Copyright (c) 2015, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include <ucg_cmd_parse.h>
#include <ucg_cmd_parse_num.h>
#include <ucg_cmd_parse_string.h>
#include <ucg_cmd.h>

/**********************************************************/

struct cmd_hello_result {
	ucg_cmd_fixed_string_t hello;
	ucg_cmd_fixed_string_t name;
};

static void cmd_hello_parsed(void *parsed_result,
	struct ucg_cmd *cl, void *data)
{
	struct cmd_hello_result *res = parsed_result;

	(void)data;
	ucg_cmd_printf(cl, "hello %s\n", res->name);
}

static ucg_cmd_tk_string_t cmd_hello_hello =
	UCG_CMD_TK_STRING(struct cmd_hello_result, hello, "hello");

static ucg_cmd_tk_string_t cmd_hello_name =
	UCG_CMD_TK_STRING(struct cmd_hello_result, name, NULL);

static ucg_cmd_inst_t cmd_hello = {
	.f = cmd_hello_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = "Say hello",
	.tokens = {        /* token list, NULL terminated */
		(void *)&cmd_hello_hello,
		(void *)&cmd_hello_name,
		NULL,
	},
};

/**********************************************************/

struct cmd_hello_tutu_result {
	ucg_cmd_fixed_string_t hello;
	ucg_cmd_fixed_string_t name1;
	ucg_cmd_fixed_string_t name2;
};

static void cmd_hello_tutu_parsed(void *parsed_result,
	struct ucg_cmd *cl, void *data)
{
	struct cmd_hello_tutu_result *res = parsed_result;

	(void)data;
	ucg_cmd_printf(cl, "hello %s and %s\n", res->name1, res->name2);
}

static ucg_cmd_tk_string_t cmd_hello_tutu_hello =
	UCG_CMD_TK_STRING(struct cmd_hello_tutu_result, hello, "hello");

static ucg_cmd_tk_string_t cmd_hello_tutu_name1 =
	UCG_CMD_TK_STRING(struct cmd_hello_tutu_result, name1, "tutu");

static ucg_cmd_tk_string_t cmd_hello_tutu_name2 =
	UCG_CMD_TK_STRING(struct cmd_hello_tutu_result, name2, NULL);

static ucg_cmd_inst_t cmd_hello_tutu = {
	.f = cmd_hello_tutu_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = "Say hello to tutu and someone else",
	.tokens = {        /* token list, NULL terminated */
		(void *)&cmd_hello_tutu_hello,
		(void *)&cmd_hello_tutu_name1,
		(void *)&cmd_hello_tutu_name2,
		NULL,
	},
};

/**********************************************************/

struct cmd_hello_toto_result {
	ucg_cmd_fixed_string_t hello;
	ucg_cmd_fixed_string_t name;
	uint16_t count;
};

static void cmd_hello_toto_parsed(void *parsed_result,
	struct ucg_cmd *cl, void *data)
{
	uint16_t i;
	struct cmd_hello_toto_result *res = parsed_result;

	(void)data;
	for (i = 0; i < res->count; i++)
		ucg_cmd_printf(cl, "hello %s\n", res->name);
}

static ucg_cmd_tk_string_t cmd_hello_toto_hello =
	UCG_CMD_TK_STRING(struct cmd_hello_toto_result, hello, "hello");

static ucg_cmd_tk_string_t cmd_hello_toto_name =
	UCG_CMD_TK_STRING(struct cmd_hello_toto_result, name, "toto#titi");

static ucg_cmd_tk_num_t cmd_hello_toto_count =
	UCG_CMD_TK_NUM(struct cmd_hello_toto_result, count, UINT16);

static ucg_cmd_inst_t cmd_hello_toto = {
	.f = cmd_hello_toto_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = "Say hello to toto or titi several times",
	.tokens = {        /* token list, NULL terminated */
		(void *)&cmd_hello_toto_hello,
		(void *)&cmd_hello_toto_name,
		(void *)&cmd_hello_toto_count,
		NULL,
	},
};

/****** CONTEXT (list of instruction) */

ucg_cmd_ctx_t main_ctx = {
	.name = "main",
	.insts = {
		&cmd_hello,
		&cmd_hello_tutu,
		&cmd_hello_toto,
		NULL,
	},
};
