/*
 * Copyright 2015, Olivier MATZ <zer0@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <ucg_delay.h>
#include <ucg_irq.h>

#include <stdio.h>
#include <inttypes.h>
#include <unistd.h>
#include <string.h>

#include "uart.h"

#if defined(__ARM_EABI__)
#include <stm32f4xx.h>

static void led_on(void)
{
	GPIOD->ODR |= (1 << 13);
}

static void led_off(void)
{
	GPIOD->ODR &= (~(1 << 13));
}

static void target_init(void)
{
	/* enable the clock to GPIOD, and stall instruction pipeline as per
	 * errata 2.1.13 "Delay after an RCC peripheral clock enabling" */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	__asm("dsb");

	/* set pin 13 to be general purpose output */
	GPIOD->MODER = (1 << 26);
}
#elif defined(__AVR__)
#include <avr/io.h>

static void led_on(void)
{
	PORTB |= (1 << 5);
}

static void led_off(void)
{
	PORTB &= (~(1 << 5));
}

static void target_init(void)
{
	DDRB = (1 << 5);
}
#endif

static void big_print(void)
{
	printf( "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
		"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
		"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
		"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\n");
}

int main(void)
{
	unsigned i;
	char c;
	int ret;

	target_init();

	/* toggle the pin */
	for (i = 0; i < 3; i++) {
		led_on();
		ucg_delay_ms(500);
		led_off();
		ucg_delay_ms(500);
	}

	uart_init();
	ucg_irq_unlock();

	big_print();

	while (1) {
		ret = fread(&c, 1, 1, stdin);
		if (ret == 1) {
			if (c == 'x')
				big_print();
			else
				fwrite(&c, 1, 1, stdout);
		} else if (ret == 0)
			clearerr(stdin);
	}

	return 0;
}
