/*
 * Copyright (c) 2009-2015, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*-
 * Copyright (c) <2010>, Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither the name of Intel Corporation nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UCG_CMD_PARSE_IPADDR_H_
#define UCG_CMD_PARSE_IPADDR_H_

#include "ucg_cmd_parse.h"

#define UCG_CMD_IPADDR_V4      0x01
#define UCG_CMD_IPADDR_V6      0x02
#define UCG_CMD_IPADDR_NETWORK 0x04

struct ucg_cmd_ipaddr {
	uint8_t family;
	union {
		struct in_addr ipv4;
		struct in6_addr ipv6;
	} addr;
	unsigned int prefixlen; /* in case of network only */
};
typedef struct ucg_cmd_ipaddr ucg_cmd_ipaddr_t;

struct ucg_cmd_tk_ipaddr_data {
	uint8_t flags;
};

struct ucg_cmd_tk_ipaddr {
	struct ucg_cmd_tk_hdr hdr;
	struct ucg_cmd_tk_ipaddr_data ipaddr_data;
};
typedef struct ucg_cmd_tk_ipaddr ucg_cmd_tk_ipaddr_t;

extern struct ucg_cmd_tk_ops ucg_cmd_tk_ipaddr_ops;

#define UCG_CMD_TK_IPADDR(structure, field)	    \
{							    \
	.hdr = {					    \
		.ops = &ucg_cmd_tk_ipaddr_ops,	    \
		.offset = offsetof(structure, field),	    \
	},						    \
	.ipaddr_data = {				    \
		.flags = UCG_CMD_IPADDR_V4 |	    \
		UCG_CMD_IPADDR_V6,			    \
	},						    \
}

#define UCG_CMD_TK_IPV4(structure, field)	    \
{							    \
	.hdr = {					    \
		.ops = &ucg_cmd_tk_ipaddr_ops,	    \
		.offset = offsetof(structure, field),	    \
	},						    \
	.ipaddr_data = {				    \
		.flags = UCG_CMD_IPADDR_V4,		    \
	},						    \
}

#define UCG_CMD_TK_IPV6(structure, field)	    \
{							    \
	.hdr = {					    \
		.ops = &ucg_cmd_tk_ipaddr_ops,	    \
		.offset = offsetof(structure, field),	    \
	},						    \
	.ipaddr_data = {				    \
		.flags = UCG_CMD_IPADDR_V6,		    \
	},						    \
}

#define UCG_CMD_TK_IPNET(structure, field)	    \
{							    \
	.hdr = {					    \
		.ops = &ucg_cmd_tk_ipaddr_ops,	    \
		.offset = offsetof(structure, field),	    \
	},						    \
	.ipaddr_data = {				    \
		.flags = UCG_CMD_IPADDR_V4 |	    \
		UCG_CMD_IPADDR_V6 |			    \
		UCG_CMD_IPADDR_NETWORK,		    \
	},						    \
}

#define UCG_CMD_TK_IPV4NET(structure, field)	    \
{							    \
	.hdr = {					    \
		.ops = &ucg_cmd_tk_ipaddr_ops,	    \
		.offset = offsetof(structure, field),	    \
	},						    \
	.ipaddr_data = {				    \
		.flags = UCG_CMD_IPADDR_V4 |	    \
		UCG_CMD_IPADDR_NETWORK,		    \
	},						    \
}

#define UCG_CMD_TK_IPV6NET(structure, field)	    \
{							    \
	.hdr = {					    \
		.ops = &ucg_cmd_tk_ipaddr_ops,	    \
		.offset = offsetof(structure, field),	    \
	},						    \
	.ipaddr_data = {				    \
		.flags = UCG_CMD_IPADDR_V4 |	    \
		UCG_CMD_IPADDR_NETWORK,		    \
	},						    \
}

#endif /* UCG_CMD_PARSE_IPADDR_H_ */
