/*
 * Copyright (c) 2009-2015, Olivier MATZ <zer0@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*-
 * Copyright (c) <2010>, Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither the name of Intel Corporation nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UCG_CMD_VT100_H_
#define UCG_CMD_VT100_H_

#define ucg_vt100_bell         "\007"
#define ucg_vt100_bs           "\010"
#define ucg_vt100_bs_clear     "\010 \010"
#define ucg_vt100_tab          "\011"
#define ucg_vt100_crnl         "\012\015"
#define ucg_vt100_clear_right  "\033[0K"
#define ucg_vt100_clear_left   "\033[1K"
#define ucg_vt100_clear_down   "\033[0J"
#define ucg_vt100_clear_up     "\033[1J"
#define ucg_vt100_clear_line   "\033[2K"
#define ucg_vt100_clear_screen "\033[2J"
#define ucg_vt100_up_arr       "\033\133\101"
#define ucg_vt100_down_arr     "\033\133\102"
#define ucg_vt100_right_arr    "\033\133\103"
#define ucg_vt100_left_arr     "\033\133\104"
#define ucg_vt100_multi_right  "\033\133%uC"
#define ucg_vt100_multi_left   "\033\133%uD"
#define ucg_vt100_suppr        "\033\133\063\176"
#define ucg_vt100_home         "\033M\033E"
#define ucg_vt100_word_left    "\033\142"
#define ucg_vt100_word_right   "\033\146"

/* Result of parsing : it must be synchronized with
 * ucg_cmd_vt100_commands[] in vt100.c */
#define UCG_CMD_KEY_UP_ARR 0
#define UCG_CMD_KEY_DOWN_ARR 1
#define UCG_CMD_KEY_RIGHT_ARR 2
#define UCG_CMD_KEY_LEFT_ARR 3
#define UCG_CMD_KEY_BKSPACE 4
#define UCG_CMD_KEY_RETURN 5
#define UCG_CMD_KEY_CTRL_A 6
#define UCG_CMD_KEY_CTRL_E 7
#define UCG_CMD_KEY_CTRL_K 8
#define UCG_CMD_KEY_CTRL_Y 9
#define UCG_CMD_KEY_CTRL_C 10
#define UCG_CMD_KEY_CTRL_F 11
#define UCG_CMD_KEY_CTRL_B 12
#define UCG_CMD_KEY_SUPPR 13
#define UCG_CMD_KEY_TAB 14
#define UCG_CMD_KEY_CTRL_D 15
#define UCG_CMD_KEY_CTRL_L 16
#define UCG_CMD_KEY_RETURN2 17
#define UCG_CMD_KEY_META_BKSPACE 18
#define UCG_CMD_KEY_WLEFT 19
#define UCG_CMD_KEY_WRIGHT 20
#define UCG_CMD_KEY_HELP 21
#define UCG_CMD_KEY_CTRL_W 22
#define UCG_CMD_KEY_CTRL_P 23
#define UCG_CMD_KEY_CTRL_N 24
#define UCG_CMD_KEY_META_D 25

extern const char *ucg_cmd_vt100_commands[];

enum ucg_cmd_vt100_parser_state {
	UCG_CMD_VT100_INIT,
	UCG_CMD_VT100_ESCAPE,
	UCG_CMD_VT100_ESCAPE_CSI
};

#define UCG_CMD_VT100_BUF_SIZE 8
struct ucg_cmd_vt100 {
	uint8_t bufpos;
	char buf[UCG_CMD_VT100_BUF_SIZE];
	enum ucg_cmd_vt100_parser_state state;
};

/**
 * Init
 */
void ucg_vt100_init(struct ucg_cmd_vt100 *vt);

#define UCG_VT100_STD_CHAR     -1
#define UCG_VT100_NOT_COMPLETE -2
/**
 * Input a new character.
 * Return UCG_VT100_STD_CHAR if the character is not part of a control sequence
 * Return UCG_VT100_NOT_COMPLETE if c is not the last char of a control sequence
 * Else return the index in ucg_vt100_commands[]
 */
int ucg_vt100_parser(struct ucg_cmd_vt100 *vt, char c);

#endif /* UCG_CMD_VT100_H_ */
