/*
 * Copyright (c) 2009-2015, Olivier MATZ <zer0@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*-
 * Copyright (c) <2010>, Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * - Neither the name of Intel Corporation nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <inttypes.h>
#include <fcntl.h>
#ifdef UCG_CMD_HAVE_TERMIOS
#include <termios.h>
#endif

#ifdef UCG_CMD_HAVE_SOCKET
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/un.h>
#endif

#include "ucg_cmd_parse.h"
#include "ucg_cmd_rdline.h"
#include "ucg_cmd_socket.h"
#include "ucg_cmd.h"

#ifdef UCG_CMD_HAVE_SOCKET
int
ucg_cmd_tcpv4_listen(in_addr_t addr, uint16_t port)
{
	int s;
	struct sockaddr_in sin_ci;
	int optval = 1;

	s = socket(PF_INET, SOCK_STREAM, 0);
	if (s < 0)
		return s;

	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
			&optval, sizeof(optval)) == -1)
		goto end;

	memset(&sin_ci, 0, sizeof(sin_ci));
	sin_ci.sin_family = AF_INET;
	sin_ci.sin_addr.s_addr = addr;
	sin_ci.sin_port = htons(port);
#ifndef __linux__
	sin_ci.sin_len = sizeof(sin_ci);
#endif
	if (bind(s, (struct sockaddr *)&sin_ci, sizeof(sin_ci)) < 0)
		goto end;

	if (listen(s, 1) < 0)
		goto end;

	return s;
 end:
	close(s);
	return -1;
}

int
ucg_cmd_tcpv6_listen(const struct in6_addr *addr6, uint16_t port)
{
	int s;
	struct sockaddr_in6 sin6_ci;

	s = socket(PF_INET6, SOCK_STREAM, 0);
	if (s < 0)
		return s;

	bzero(&sin6_ci, sizeof(sin6_ci));
	sin6_ci.sin6_family = AF_INET6;
#ifndef __linux__
	sin6_ci.sin6_len = sizeof(sin6_ci);
#endif
	memcpy(&sin6_ci.sin6_addr, addr6, sizeof(sin6_ci.sin6_addr));
	sin6_ci.sin6_port = htons(port);
	if (bind(s, (struct sockaddr *) &sin6_ci, sizeof(sin6_ci)) < 0)
		goto end;

	if (listen(s, 1) < 0)
		goto end;

	return s;
 end:
	close(s);
	return -1;
}

int
ucg_cmd_unix_listen(const char *filename)
{
	int s;
	struct sockaddr_un servAddr;

	s = socket(AF_UNIX, SOCK_STREAM, 0);
	if (s < 0)
		return s;

	bzero(&servAddr, sizeof(servAddr));
	servAddr.sun_family = AF_UNIX;
	memcpy(servAddr.sun_path, filename , strlen(filename));

	unlink(filename);
	if(bind(s, (struct sockaddr *) &servAddr, sizeof(servAddr)) < 0)
		goto end;

	if (listen(s, 1) < 0)
		goto end;

	return s;
 end:
	close(s);
	return -1;
}

struct ucg_cmd *
ucg_cmd_accept(ucg_cmd_ctx_t *ctx, const char *prompt, int s)
{
	FILE *f;
	int s2;
	struct sockaddr sin;
	socklen_t sinlen;

	sinlen = sizeof(struct sockaddr);

	if ((s2 = accept(s, &sin, &sinlen)) < 0)
		return NULL;

	f = fdopen(s2, "r+");
	if (f == NULL) {
		close(s2);
		return NULL;
	}
	return (ucg_cmd_new(ctx, prompt, f, f));
}
#endif
