/*
 * Copyright (c) 2015, Olivier MATZ <zer0@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdint.h>
#include <string.h>
#ifdef UCG_CMD_HAVE_TERMIOS
#include <termios.h>
#endif

#include "ucg_cmd.h"
#include "ucg_cmd_termios.h"

int ucg_cmd_termios_raw(struct ucg_cmd *cl)
{
	int ret = 0;

#ifdef UCG_CMD_HAVE_TERMIOS
	struct termios oldterm, term;

	ret = tcgetattr(0, &oldterm);
	if (ret < 0)
		return ret;

	memcpy(&term, &oldterm, sizeof(term));
	term.c_lflag &= ~(ICANON | ECHO | ISIG);
	ret = tcsetattr(0, TCSANOW, &term);
	if (ret < 0)
		return ret;

	memcpy(&cl->oldterm, &oldterm, sizeof(term));
#endif

	(void)cl; /* silent compiler */

	return ret;
}

int ucg_cmd_termios_restore(struct ucg_cmd *cl)
{
	int ret = 0;
	(void)cl; /* silent compiler */

#ifdef UCG_CMD_HAVE_TERMIOS
	ret = tcsetattr(fileno(stdin), TCSANOW, &cl->oldterm);
#endif
	return ret;
}

