/*
 * Copyright (c) 2010, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/queue.h>

#include "cfzy_confnode.h"

/* write config value in file f in a dotconfig-like manner. Return 0
 * on success. */
static int menu_dotconfig_write(const struct cfzy_confnode *n, FILE *f)
{
	const struct cfzy_confnode *c;

	if (fprintf(f,
		    "#\n"
		    "# -- %s\n"
		    "#\n", n->prompt) < 0)
		return -1;

	/* dump children */
	TAILQ_FOREACH(c, &n->children, child_next) {
		if (cfzy_confnode_dotconfig_write(c, f) < 0)
			return -1;
	}

	return 0;
}

/* this node is always enabled whatever the value */
static int menu_str2bool(const struct cfzy_confnode *n,
			      const char *value)
{
	(void)n;

	/* should not happen */
	if (value != NULL)
		return -1;

	return 1;
}

/* Return a string identifying the node type ("config", "menuconfig",
 * "choice", ...) */
static const char *menu_get_type_str(const struct cfzy_confnode *n)
{
	(void)n;
	return "menu";
}

static struct cfzy_confnode_ops menu_ops = {
	.free = NULL,
	.add_attr = NULL,
	.close = NULL,
	.str2bool = menu_str2bool,
	.dotconfig_write = menu_dotconfig_write,
	.c_hdr_write = NULL,
	.get_type_str = menu_get_type_str,
	.set_uservalue = NULL,
};

/* Create a new node */
enum cfzy_parse_return cfzy_confnode_menu_new(struct cfzy_confnode *n,
					      const struct cfzy_token_list *tklist)
{
	struct cfzy_token *tok;
	tok = TAILQ_FIRST(&tklist->list);

	if (strcmp(tok->str, "menu") || tklist->n_token != 2)
		return NO_MATCH;

	tok = TAILQ_NEXT(tok, next);
	n->ops = &menu_ops;
	n->flags = CFZY_F_IS_DIR |
		CFZY_F_NO_SET_DEPS |
		CFZY_F_NO_SET_DEFAULT |
		CFZY_F_NO_SET_VALUE |
		CFZY_F_DOTCONF_SHOW_TITLE;

	n->name = strdup(tok->str);
	if (n->name == NULL)
		return ERROR;

	return SUCCESS;
}
