/*
 * Copyright (c) 2010, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/queue.h>

#include "cfzy_confnode.h"

/* this node is always enabled whatever the value */
static int root_str2bool(const struct cfzy_confnode *n,
			      const char *value)
{
	(void)n;

	/* should not happen */
	if (value != NULL)
		return -1;

	return 1;
}

static struct cfzy_confnode_ops root_ops = {
	.free = NULL,
	.add_attr = NULL,
	.close = NULL,
	.str2bool = root_str2bool,
	.c_hdr_write = NULL,
	.dotconfig_write = NULL,
	.get_type_str = NULL,
	.set_uservalue = NULL,
};

/* Create a new root node */
struct cfzy_confnode *cfzy_confnode_root_new(struct cfzy_conftree *conftree)
{
	struct cfzy_confnode *n;

	n = cfzy_confnode_alloc(conftree);
	if (n == NULL)
		return NULL;

	n->ops = &root_ops;
	n->flags = CFZY_F_IS_ROOT |
		CFZY_F_INVISIBLE |
		CFZY_F_NO_SET_VALUE |
		CFZY_F_NO_NAME |
		CFZY_F_NO_SET_DEFAULT |
		CFZY_F_NO_SET_DEPS;

	n->name = strdup("root");
	if (n->name == NULL)
		return NULL;

	return n;
}
