/*
 * Copyright (c) 2013, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file
 * Confizery dotconfig parsing/generation
 *
 * This modules provides an API to parse or generate a .config file
 * from a configuration tree.
 */

#ifndef _CFZY_DOTCONFIG_H_
#define _CFZY_DOTCONFIG_H_

/**
 * Write configuration in a .config file
 *
 * Write the configuration values (.config like) in a file. This file
 * is designed to be sourced in a shell script or in a makefile.
 *
 * If the file already exists and has the same content, nothing is
 * written in it.
 *
 * @param conftree
 *   the configuration tree
 * @param filename
 *   name of the output file
 * @return
 *   0 on success, negative on error
 */
int cfzy_dotconfig_write(const struct cfzy_conftree *conftree,
			 const char *filename);

/**
 * Read and parse a dotconfig configuration file
 *
 * Read the configuration values (.config like) from a file, and set
 * the values in the configuration tree.
 *
 * @param conftree
 *   the configuration tree
 * @param filename
 *   name of the output file
 * @return
 *   0 on success, negative on error
 */
int cfzy_dotconfig_read(struct cfzy_conftree *conftree, const char *filename);

#endif
