#!/usr/bin/env python

import matplotlib.pyplot as plt
import networkx as nx
import sys, re, random

G = nx.Graph()
labels = {}
levels = {}
init_pos = {}
maxlevel = 0

# always keep the same seed
random.seed(1234)

# parse lines, new vertice is:
#   vertice LEVEL OPERATOR:VERTICE_NAME
# new edge is:
#   edge OPERATOR:VERTICE_NAME OPERATOR:VERTICE_NAME
while True:
    l = sys.stdin.readline()
    if l == "":
        break
    m = re.match("^vertice ([0-9]+) ([^\s]+)$", l)
    if m:
        level = m.groups()[0]
        level = int(level)
        operator, name = m.groups()[1].split(":")
        print "vert: operator = %s, level = %d, name = %s"%(operator, level, name)
        labels[name] = operator
        init_pos[name] = (random.random(), -level)
        levels[name] = level
        if level > maxlevel:
            maxlevel = level
        continue
    m = re.match("^edge ([^\s]+) ([^\s]+)$", l)
    if m:
        vertice1, vertice2 = m.groups()
        G.add_edge(vertice1, vertice2)
        continue

pos = nx.spring_layout(G, pos=init_pos)

# root node is blue and bigger
rootnode = filter(lambda x:levels[x] == 0, levels.keys())
nx.draw_networkx_nodes(G, pos, node_color='b', node_size=2000, nodelist=rootnode)

# graph other nodes
for level in range(1, maxlevel + 1):
    othernodes = filter(lambda x:levels[x] == level, levels.keys())
    color = (1., float(level)/maxlevel, float(level)/maxlevel)
    color = [color] * len(othernodes)
    nx.draw_networkx_nodes(G, pos, node_color=color, node_size=1000, nodelist=othernodes)

nx.draw_networkx_edges(G, pos)
nx.draw_networkx_labels(G, pos, labels, font_size=20, font_family='sans-serif')

plt.axis('off')
plt.savefig("weighted_graph.png") # save as png
plt.show() # display
