/*
 * Copyright (c) 2013, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include "cfzy_list.h"

/* allocate a list_elt and insert at the tail of the list */
int cfzy_list_add_tail(struct cfzy_list_head *list, void *ptr)
{
	struct cfzy_list_elt *e;

	e = malloc(sizeof(struct cfzy_list_elt));
	if (e == NULL)
		return -1;

	e->ptr = ptr;
	TAILQ_INSERT_TAIL(list, e, next);
	return 0;
}

/* allocate a list_elt and insert at the tail of the list */
int cfzy_list_add_head(struct cfzy_list_head *list, void *ptr)
{
	struct cfzy_list_elt *e;

	e = malloc(sizeof(struct cfzy_list_elt));
	if (e == NULL)
		return -1;

	e->ptr = ptr;
	TAILQ_INSERT_HEAD(list, e, next);
	return 0;
}

/* remove from the list and free the list_elt */
void cfzy_list_del(struct cfzy_list_head *list, struct cfzy_list_elt *e)
{
	TAILQ_REMOVE(list, e, next);
	free(e);
}

/* check if an elt is in list */
int cfzy_list_elt_is_in_list(const struct cfzy_list_head *list, const void *ptr)
{
	struct cfzy_list_elt *e;

	TAILQ_FOREACH(e, list, next) {
		if (e->ptr == ptr)
			return 1;
	}
	return 0;
}

/* allocate a new list_head */
struct cfzy_list_head *cfzy_list_alloc(void)
{
	struct cfzy_list_head *list;

	list = malloc(sizeof(struct cfzy_list_head));
	if (list == NULL)
		return NULL;

	TAILQ_INIT(list);
	return list;
}

/* empty the list, freeing the list_elt, and free the list_head */
void cfzy_list_free(struct cfzy_list_head *list, void (*free_fct)(void *))
{
	struct cfzy_list_elt *e;

	while ((e = TAILQ_FIRST(list)) != NULL) {
		TAILQ_REMOVE(list, e, next);
		if (free_fct != NULL)
			free_fct(e->ptr);
		free(e);
	}
	free(list);
}
