/*
 * Copyright (c) 2013, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this log of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this log of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _CFZY_LOG_H_
#define _CFZY_LOG_H_

/**
 * @file
 * Confizery Log
 *
 * This module provides logging helpers. XXX explain
 */

/* log levels */
#define CFZY_LOG_ERR      0
#define CFZY_LOG_WARNING  1
#define CFZY_LOG_NOTICE   2
#define CFZY_LOG_INFO     3
#define CFZY_LOG_DEBUG    4

/**
 * Initialize log subsystem
 *
 * @return
 *   0 on succes, -1 on error
 */
int cfzy_log_init(void);

/**
 * Uninitialize log subsystem
 */
void cfzy_log_exit(void);

/**
 * Set default log level
 *
 * @param level
 *   Level of log, -1 means no log at all
 * @return
 *   0 on succes, -1 on error
 */
int cfzy_log_set_default_level(int level);

/**
 * Set log level for a specific log type
 *
 * @param logtype
 *   String describing the log type
 * @param level
 *   Level of log, -1 means no log at all
 * @return
 *   0 on succes, -1 on error
 */
int cfzy_log_set_level(const char *logtype, int level);

/**
 * Print a log if given level is <= to configured log level
 *
 * The log is displayed on standard error.
 *
 * @param logtype
 *   String describing the log type
 * @param level
 *   Level at which the text must be displayed
 * @param fmt
 *   Format string, followed by optional arguments, like in printf.
 * @return
 *   The number of written characters (0 if nothing is diplayed),
 *   or -1 on error.
 */
int cfzy_log_printf(const char *logtype, int level, const char *fmt, ...);

#if 0
/**
 * Print a log if given level is <= to configured log level
 */
#define CFZY_LOG(logtype, level, fmt, args...)				\
	cfzy_log_printf(logtype, CFZY_LOG_##level, fmt,			\
			## args)
#else
#define CFZY_LOG(logtype, level, fmt, args...)				\
	cfzy_log_printf(logtype, CFZY_LOG_##level, "%s():%d " fmt,	\
			__func__, __LINE__, ## args)
#endif

#endif /* _CFZY_LOG_H_ */
